'use strict';

const mysql = require('mysql');
const async = require('async');

const createPool = (_options, dbname) => {

    _options.database = dbname ? dbname : _options.database; //数据库名的检测
    let options = _options || {
        host: '',
        user: '',
        password: '',
        database: ''
    };

    const pool = mysql.createPool(options);

    const getConnection = () => {
        return new Promise((resolve, reject) => {
            pool.getConnection((err, conn) => {
                if (err) {
                    return reject(err);
                }
                return resolve(conn);
            });
        });
    }; 

    const beginTransaction = (conn) => {
        return new Promise((resolve, reject) => {
            conn.beginTransaction(err => {
                if (err) {
                    return reject(err);
                }
                return resolve();
            });
        });
    };

    const commit = (conn) => {
        return new Promise((resolve, reject) => {
            conn.commit(err => {
                if (err) {
                    conn.rollback(() => {
                        conn.release();
                        return reject();
                    })
                }
                conn.release();
                return resolve();
            })
        });
    };

    const rollback = (conn) => {
        return new Promise((resolve, reject) => {
            conn.rollback(() => {
                conn.release();
            });
            return resolve();
        });
    };

    const query = (sql, values, conn) => {
        return new Promise((resolve, reject) => {
            if (conn) {
                if (values && values.length) {
                    conn.query(sql, values, (err, rows, fields) => {
                        if (err) {
                            reject(err);
                        } else {
                            resolve(rows, fields);
                        }
                    });
                } else {
                    conn.query(sql, (err, rows, fields) => {
                        if (err) {
                            console.log('error:>>>> ', err);
                            reject(err);
                        } else {
                            resolve(rows, fields);
                        }
                    });
                }
            } else {
                pool.getConnection((err, connection) => {
                    if (err) {
                        reject(err);
                    } else {
                        if (values && values.length) {
                            connection.query(sql, values, (err, rows, fields) => {
                                connection.release();
                                if (err) {
                                    reject(err);
                                } else {
                                    resolve(rows, fields);
                                }
                            });
                        } else {
                            connection.query(sql, (err, rows, fields) => {
                                connection.release();
                                if (err) {
                                    console.log('error:>>>> ', err);
                                    reject(err);
                                } else {
                                    resolve(rows, fields);
                                }
                            });
                        }
                    }
                });
            }
        });
    };

    const findOne = (sql, values, db) => {
        return new Promise((resolve, reject) => {
            pool.getConnection((err, connection) => {
                if (err) {
                    reject(err);
                } else {
                    if (values && values.length) {
                        connection.query(sql, values, (err, rows, fields) => {
                            connection.release();
                            if (err) {
                                reject(err);
                            } else {
                                resolve(rows[0] ? rows[0] : {}, fields);
                            }
                        });
                    } else {
                        connection.query(sql, (err, rows, fields) => {
                            connection.release();
                            if (err) {
                                console.log('error:>>>> ', err);
                                reject(err);
                            } else {
                                resolve(rows[0] ? rows[0] : {}, fields);
                            }
                        });
                    }
                }
            });
        });
    };

    const end = () => {
        return new Promise((resolve, reject) => {
            pool.end(function (err) {
                if (err) {
                    reject(err);
                } else {
                    resolve();
                }
            });
        });
    };

    const transaction = (sqlsAndValues) => {
        return new Promise((resolve, reject) => {
            if (!Array.isArray(sqlsAndValues)) {
                return reject('params error');
            }
            pool.getConnection((err, connection) => {
                if (err) {
                    reject(err);
                } else {
                    connection.beginTransaction((err) => {
                        if (err) {
                            reject(err);
                        }
                        let funs = [];
                        sqlsAndValues.forEach((sv) => {
                            let fun = (callback) => {
                                connection.query(sv.sql, sv.values, callback);
                            };
                            funs.push(fun);
                        });

                        async.series(funs, (err, result) => {
                            if (err) {
                                connection.rollback(() => {
                                    connection.release();
                                    reject(err);
                                });
                            } else {
                                connection.commit((err) => {
                                    if (err) {
                                        return connection.rollback(() => {
                                            connection.release();
                                            reject(err);
                                        });
                                    } else {
                                        connection.release();
                                        resolve();
                                    }
                                });
                            }
                        });
                    });
                }
            });
        });
    };

    return {
        end: end,
        pool: pool,
        query: query,
        commit: commit,
        findOne: findOne,
        rollback: rollback,
        transaction: transaction,
        getConnection: getConnection,
        beginTransaction: beginTransaction,
    };
};

module.exports = createPool;